using System;
using System.Collections.Generic;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using Managers;
#region Signature
/*
 * This file was written by Daniel Selnick
 * email: d.selnick@gmail.com
 * It is under the following license
 *  http://www.opensource.org/licenses/ms-pl.html
 */
#endregion

namespace PrototypeGame
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //managers for the game components
        BulletManager shotManager;
        TowerManager towerManager;
        PlayerManager playerManager;
        EnemyManager enemyManager;
        CollisionManager collisionManager;

        public const int gameScreenWidth = 1024;
        public const int gameScreenHeight = 768;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            //make the game fullscreen
            //graphics.IsFullScreen = true;
            graphics.PreferredBackBufferHeight = gameScreenHeight;
            graphics.PreferredBackBufferWidth = gameScreenWidth;
            
            Content.RootDirectory = "Content";
           
        }
        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            shotManager = new BulletManager(this);

            towerManager = new TowerManager(this);

            playerManager = new PlayerManager(this);

            enemyManager = new EnemyManager(this);

            //collisionManager = new CollisionManager(this);


            this.Components.Add(shotManager);
            this.Components.Add(towerManager);
            this.Components.Add(playerManager);
            this.Components.Add(enemyManager);
            //this.Components.Add(collisionManager);

            this.IsMouseVisible = true;

            
            base.Initialize();
        }
        
        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            
            spriteBatch = new SpriteBatch(GraphicsDevice);
            base.LoadContent();
            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        /// 
        
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            base.Draw(gameTime);
        }
    }
}
